#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <string.h>

#include "signal.h"
#include "output.h"
#include "main.h"
#include "config.h"
#include "checkport.h"

/* Let this point at the function we wish to use for portchecking */
int (*checkport_p)(unsigned long host, unsigned short port)=&checkport_connect;

int checkport(unsigned long host, unsigned short port)
{
  int status;
  
#ifdef DEBUG
  fprintf(stderr, "CHECKING: %s:%i\n", inet_ntoa(*(struct in_addr *)&host), ntohs(port));
#endif
  
  if (port==0) /* we shouldnt scan this one.. */
    return 0;

  if (options.verbose >= 1)
    output_current(host, ntohs(port));

  options.info.count++;

  signal(SIGALRM, (void *)&sigalrm);

  /* multithreading support */
  if (options.thread.thread)
    {
      while ((options.thread.spawned == MAXTHREAD) || (options.thread.spawned >= options.thread.thread)) usleep(MAXTHREAD_DELAY);
      options.parent=0;

      if ((options.thread.pid=fork()) != 0)
	{ /* Parent */
	  signal(SIGCHLD, (void *)&sigchld);
	  options.thread.spawned++;
	  options.parent=1;
	  sleep(options.delay);
	  return 0;
	}
    }
  
  if (options.checkport)
    status = checkport_p(host,port);
  else
    status = 1; /* Force open, to use as some kinda frontendthingy */
  
  sleep(options.delay);

  report(host, port, status);
  if (options.thread.thread) { shmdt((char *)options.odata); exit(0); } /* If we are a child, lets kill ourself */

  return status;
}

int checkport_connect(unsigned long host, unsigned short port)
{
  int status;
  struct sockaddr_in addr;
  int sock;
  
  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
    {
      perror("socket");
      exit(255);
    }

  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = host;
  addr.sin_port = port;
  
  /* Set timeout value */
  if (options.timeout)
    alarm(options.timeout);
  
  status = connect(sock, (struct sockaddr *)&addr, sizeof(struct sockaddr_in));
  alarm(0);
    
  close(sock);

  return !status;
}


/* This is not very reliable at all.. its up to the remote peer
   to decide whether they should answer or not, i got named
   to answer on udp port 53 with this function anyway..
   But never trust this to be completely true,
   besides.. udp packets may be lost on the net..
   i do not use any mechanism at all to recheck or anything */
int checkport_udp(unsigned long host, unsigned short port)
{
  int status;
  struct sockaddr_in addr;
  int sock;
  char buffer[512];
  int size=sizeof(struct sockaddr_in);

  memset(buffer, 0, sizeof(buffer));

  if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
    {
      perror("socket");
      exit(255);
    }

  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = host;
  addr.sin_port = port;
  
  /* Set timeout value */
  if (options.timeout)
    alarm(options.timeout);
  else
    alarm(DEFAULT_UDP_TIMEOUT);
  
  status = sendto(sock, buffer, sizeof(buffer), 0, (struct sockaddr *)&addr, size);
  if (status < 0) perror("sendto");

  status = recvfrom(sock, buffer, sizeof(buffer), 0, (struct sockaddr *)&addr, &size);
  if (status < 0)
    {
      switch(errno)
	{
	case EINTR: /* In case we timeout, might indicate port was open */
	  status=2; /* maybe open, or timeout */
	  break;
	  
	default:
	  status=0;
	  break;
	}
    }
  else
    status=1; /* This port was absolutely open */

  alarm(0);
  close(sock);

  return status;
}
