#include <getopt.h>
#include <limits.h>
#include <string.h>
#include <signal.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <fcntl.h>
#include <stdlib.h>

#include "main.h"
#include "getoptions.h"
#include "resolve.h"
#include "config.h"
#include "input.h"
#include "checkport.h"
#include "signal.h"

int getoptions(int ac, char **av)
{
  int option;

  initoptions();
  initsignals();

  if (ac < 3) { options.verbose=0xff; options.parent=0; usage(av[0]); }

  options.info.ports=initport(av[2]);

  options.cmdlinehost=av[1];
  options.cmdlineport=av[2];

  while ((option=getopt(ac, av, "DuyhiVf:va:p:P:m:s:rNnd:t:")) != EOF)
    switch(option)
      {
      case 'u':
	checkport_p=&checkport_udp;
	options.ident=0; /* no udp support what i know of */
	options.checkportverbose=!options.checkportverbose; /* if -u used 2 times, dont be verbose */
	break;

      case 'y':
	options.ident=1;
	break;

      case 'h':
	if (!options.lamein)
	  {
	    char *prog=getenv("PROG_HOST");
	    char *temp;
	    int tsize=0;

	    tsize+=strlen(options.cmdlinehost);
	    if (prog)
	      tsize+=strlen(prog);
	    else
	      tsize+=strlen(PROG_HOST);
	    
	    temp=(char *)malloc(tsize+1+6000);
	    
	    if (prog)
	      sprintf(temp, "%s -l %s", prog, options.cmdlinehost);
	    else
	      sprintf(temp, "%s -l %s", PROG_HOST, options.cmdlinehost);

	    options.lamein=popen(temp, "r");
	    if (!options.lamein)
	      {
		perror("popen");
		exit(255);
	      }
	    options.inputtype=I_PROG_HOST;

	    free(temp);
	  }
	break;

      case 'i':
	if (!options.lamein)
	  {
	    options.lamein=fopen(options.cmdlinehost, "r");
	    options.inputtype=I_FILE;
	  }
	break;

      case 'V':
	options.nodefaultoutput=1;
	break;

      case 'f':
	fclose(options.lameout);
	options.lameout = fopen(optarg, "w");
	break;

      case 'v':
	options.verbose++;
	break;

      case 'a':
	options.thread.thread=strtol(optarg, NULL, 0);
	if (options.thread.thread > MAXTHREAD) { fprintf(stderr, "Too much multithreading!\n"); exit(255); }
	options.thread.shkey = shmget(0, sizeof(struct odata), IPC_CREAT | 0600);
	options.odata = (struct odata *)shmat(options.thread.shkey, 0, 0);
	break;

      case 'P':
	options.closedcmd=optarg;
	if (options.verbose < 3) options.verbose = 3;
	break;
	
      case 'p':
	options.opencmd=optarg;
	break;
	
      case 'm':
	options.end = strtol(optarg, NULL, 0);
	break;
	
      case 's':
	options.start = strtol(optarg, NULL, 0);
	break;

      case 'r':
	options.random=1;
	break;

      case 'n':
	options.dolookup=!options.dolookup;
	break;

      case 'N':
	options.ansi=!options.ansi;
	break;

      case 't':
	options.timeout=strtol(optarg, NULL, 0);
	break;

      case 'D':
	options.checkport=0;
	options.nodefaultoutput=!options.nodefaultoutput;
	break;

      case 'd':
	options.delay=strtol(optarg, NULL, 0);
	break;

      }

  return ac;
}

void usage(char *av)
{
  options.verbose=0xff;

  fprintf(stderr, "%s host port [options]\n", av);
  fprintf(stderr, "-t value                timeout in seconds.\n");
  fprintf(stderr, "-d value                delay in seconds.\n");
  fprintf(stderr, "-D                      don't try the port, just print\n");
  fprintf(stderr, "-r                      random scan.\n");
  fprintf(stderr, "-N                      turn %s ansi.\n", USE_ANSI ? "OFF" : "ON");
  fprintf(stderr, "-n                      turn %s resolving of hostnames\n", DOLOOKUP ? "OFF" : "ON");
  fprintf(stderr, "-y                      try to resolve userid of daemon\n");
  fprintf(stderr, "-p cmd                  command to run if port was open\n");
  fprintf(stderr, "-P cmd                  command to run if port was closed\n");
  fprintf(stderr, "-a value                no. of processes to multithread with (max %i)\n", MAXTHREAD);
  fprintf(stderr, "-s value                start scanning at this port\n");
  fprintf(stderr, "-m value                scan up to and including this port\n");
  fprintf(stderr, "-f name                 output to a file (except stderr)\n");
  fprintf(stderr, "-i                      host = a file containing hostnames\n");
  fprintf(stderr, "-v[vvv ... ]            add verbosity\n");
  fprintf(stderr, "-V                      no default output\n");

#ifdef PROG_HOST
  fprintf(stderr, "-h                      host is a domain, scan all hosts found in it\n");
#endif

  exithandler();
}

void initoptions(void)
{
  memset((char *)&options, 0, sizeof(struct options));

  /* everything that should not default to 0 */
  options.info.start = time(NULL);
  options.end=USHRT_MAX;
  options.ansi=USE_ANSI;
  options.dolookup=DOLOOKUP;
  options.feedstatus=1;
  options.parent=1;
  options.checkport=1;
  options.checkportverbose=1;

  options.lameout = stdout;
}
