#include <stdio.h>
#include <unistd.h>
#include <netinet/in.h>
#include <string.h>
#include <arpa/inet.h>
#include <sys/wait.h>

#include "main.h"
#include "output.h"
#include "config.h"
#include "ident.h"
#include "runcmd.h"
#include "resolve.h"

void output_current(unsigned long host, unsigned short port)
{
  static char buffer[128];
  int i;

  bzero(buffer, 128);

  if (options.thread.thread)
    while (IS_LOCKED) usleep(ODELAY);

  lock_output();
  sprintf(buffer, "Currently scanning %s:%i              ", inet_ntoa(*(struct in_addr *)&host), port);
  fprintf(stderr, "%s", buffer);
  fflush(stderr);
  for (i=0; i < strlen(buffer)+1; i++) fprintf(stderr, "\b");
  unlock_output();

  return;
}

void report(unsigned long host, unsigned short port, short status)
{
  if (status == 2 && !options.checkportverbose) status=0; /* If not checkportverbose, set status 0(closed) */
  if (!status && (options.verbose < 2)) return; /* verbose >= 2 to show CLOSED msg's */

  if (!options.thread.thread)
    output(host, port, status);
  else
    {
      while (IS_LOCKED) usleep(ODELAY);
      
      lock_output();
      output(host, port, status);
      unlock_output();
    }

  return;
}

void output(unsigned long host, unsigned short port, short status)
{
  char *command;
    
  if (!options.nodefaultoutput)
    {
      char *temp=NULL;
      
      if (options.ident)
	temp=ident_lookup(host, port);

      fprintf(stderr, "%s:%i - ", revlookup(host, options.dolookup), ntohs(port));
      
      if ((status == 2) && options.checkportverbose)
	fprintf(stderr, "Might be ");

      if (status)
	fprintf(stderr, "Open ");
      else
	fprintf(stderr, "Closed ");
      
      if (temp)
	fprintf(stderr, "[%s]", temp);
      
      /* End of the line.. */
      fprintf(stderr, "                                    \n");
    }

  /* Run command */
  if (options.opencmd || options.closedcmd)
    {
      if (status)
	command=(char *)gencmd(host,port,options.opencmd);
      else
	command=(char *)gencmd(host,port,options.closedcmd);
      
      if (command)
	{ /* system() call sucked, disabled my signals while running the commands */
	  pid_t pid;
	  
	  if ((pid = fork()) == 0)
	    execl("/bin/sh", "sh", "-c", command, NULL);
	    else
	    waitpid(pid, NULL, 0);
	}
    }

  fflush(options.lameout);

  return;
}

int lock_output(void)
{
  if (options.thread.thread)
    options.odata->locked=1;

  return 0;
}

int unlock_output(void)
{
  if (options.thread.thread)
    options.odata->locked=0;

  return 0;
}
